package de.twenty11.skysail.common.ext.dbviewer;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonProperty;

import de.twenty11.skysail.common.Presentable;
import de.twenty11.skysail.common.PresentableHeader;
import de.twenty11.skysail.common.forms.Field;
import de.twenty11.skysail.common.forms.Form;
import de.twenty11.skysail.common.forms.ValuesProviderImpl;
import de.twenty11.skysail.common.um.SkysailUser;

@Entity
@Form(name="ConnectionForm")
public class ConnectionDetails implements Presentable, Serializable {

    private static final long serialVersionUID = -1079406626730388016L;

    public static final String NAME = "name";
    public static final String USERNAME = "username";
    public static final String PASSWORD = "password";
    public static final String DRIVERNAME = "drivername";
    public static final String URL = "url";

    @Id
    @GeneratedValue
    @JsonIgnore
    private int pid;// primary key for db

    public int getPid() {
        return this.pid;
    }

    @NotNull(message = "Name is mandatory")
    @Size(min = 1, message = NAME + " must not be empty")
    @Field(valuesProvider = JdbcDriverNames.class)
    private String name;
    
    @ManyToOne
    private SkysailUser owner;

    @Field(valuesProvider=ValuesProviderImpl.class)
    private String username;
    
    @Field(valuesProvider=ValuesProviderImpl.class)
    private String password;

    @Field(valuesProvider=JdbcDriverNames.class)
    private String driverName;

    @Field(valuesProvider=ValuesProviderImpl.class)
    private String url;

    @Transient
    private Map<String, String> links;

    // @javax.persistence.Transient
    // private String schemasLink = "http://www.heise.de";

    public ConnectionDetails() {
        // needed for javax.persistence
    }

    @JsonCreator
    public ConnectionDetails(@JsonProperty("name") String name, @JsonProperty("username") String username,
            @JsonProperty("password") String password, @JsonProperty("url") String url,
            @JsonProperty("driverClassName") String driverClassName) {
        this.name = name;
        this.username = username;
        this.password = password;
        this.url = url;
        this.driverName = driverClassName;
    }

    public String getName() {
        return name;
    }

    @NotNull(message = "field '" + USERNAME + "' is mandatory")
    @Size(min = 1)
    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }

    @NotNull(message = "field '" + DRIVERNAME + "' is mandatory")
    public String getDriverName() {
        return driverName;
    }

    @NotNull(message = "field '" + URL + "' is mandatory")
    public String getUrl() {
        return url;
    }
    
    public SkysailUser getOwner() {
        return owner;
    }
    
    public void setOwner(SkysailUser owner) {
        this.owner = owner;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(name).append(", ");
        sb.append(username).append(", ");
        sb.append(url);
        return sb.toString();
    }

    public Map<String, String> toMap() {
        Map<String, String> result = new HashMap<String, String>();
        result.put(NAME, name);
        result.put(USERNAME, username);
        result.put(PASSWORD, password);
        result.put(URL, url);
        result.put(DRIVERNAME, driverName);
        return result;
    }

    @Override
    @JsonIgnore
    public PresentableHeader getHeader() {
        return new PresentableHeader.Builder(url).setLink("connections/" + name + "/schemas").setImage("icon-globe")
                .build();
    }

    @Override
    @JsonIgnore
    public Map<String, Object> getContent() {
        Map<String, Object> result = new HashMap<String, Object>();
        result.put(NAME, name);
        result.put(USERNAME, username);
        result.put(PASSWORD, "******");
        result.put(URL, url);
        result.put(DRIVERNAME, driverName);
        return result;
    }


}
